

TPREC = -4

TRACKS = (
    'Mario Kart Stadium', 'Water Park', 'Sweet Sweet Canyon', 'Thwomp Ruins',
    'Mario Circuit', 'Toad Harbor', 'Twisted Mansion', 'Shy Guy Falls',
    'Sunshine Airport', 'Dolphin Shoals', 'Electrodrome', 'Mount Wario',
    'Cloudtop Cruise', 'Bone-Dry Dunes', "Bowser's Castle", 'Rainbow Road',
    'Wii Moo Moo Meadows', 'GBA Mario Circuit', 'DS Cheep Cheep Beach', "N64 Toad's Turnpike",
    'GCN Dry Dry Desert', 'SNES Donut Plains 3', 'N64 Royal Raceway', '3DS DK Jungle',
    'DS Wario Stadium', 'GCN Sherbet Land', '3DS Music Park', 'N64 Yoshi Valley',
    'DS Tick-Tock Clock', '3DS Piranha Plant Slide', 'Wii Grumble Volcano', 'N64 Rainbow Road',
    'GCN Yoshi Circuit', 'Excitebike Arena', 'Dragon Driftway', 'Mute City',
    "Wii Wario's Gold Mine", 'SNES Rainbow Road', 'Ice Ice Outpost', 'Hyrule Circuit',
    'GCN Baby Park', 'GBA Cheese Land', 'Wild Woods', 'Animal Crossing',
    '3DS Neo Bowser City', 'GBA Ribbon Road', 'Super Bell Subway','Big Blue'
)

CUPS = {
    'Mushroom': ['Mario Kart Stadium', 'Water Park', 'Sweet Sweet Canyon', 'Thwomp Ruins'],
    'Flower': ['Mario Circuit', 'Toad Harbor', 'Twisted Mansion', 'Shy Guy Falls'],
    'Star': ['Sunshine Airport', 'Dolphin Shoals', 'Electrodrome', 'Mount Wario'],
    'Special': ['Cloudtop Cruise', 'Bone-Dry Dunes', "Bowser's Castle", 'Rainbow Road'],
    'Shell': ['Wii Moo Moo Meadows', 'GBA Mario Circuit', 'DS Cheep Cheep Beach', "N64 Toad's Turnpike"],
    'Banana': ['GCN Dry Dry Desert', 'SNES Donut Plains 3', 'N64 Royal Raceway', '3DS DK Jungle'],
    'Leaf': ['DS Wario Stadium', 'GCN Sherbet Land', '3DS Music Park', 'N64 Yoshi Valley'],
    'Lightning': ['DS Tick-Tock Clock', '3DS Piranha Plant Slide', 'Wii Grumble Volcano', 'N64 Rainbow Road'],
    'Egg': ['GCN Yoshi Circuit', 'Excitebike Arena', 'Dragon Driftway', 'Mute City'],
    'Triforce': ["Wii Wario's Gold Mine", 'SNES Rainbow Road', 'Ice Ice Outpost', 'Hyrule Circuit'],
    'Crossing': ['GCN Baby Park', 'GBA Cheese Land', 'Wild Woods', 'Animal Crossing'],
    'Bell': ['3DS Neo Bowser City', 'GBA Ribbon Road', 'Super Bell Subway','Big Blue']
}

CUPS_ORDER = (
    'Mushroom', 'Flower', 'Star', 'Special', 'Shell', 'Banana', 
    'Leaf', 'Lightning', 'Egg', 'Triforce', 'Crossing', 'Bell'
)

pad = ['Start']
CATEGORIES = {
    'Nitro': pad + list(TRACKS[:-32]),
    'Retro': ['Rainbow Road'] + list(TRACKS[-32:-16]),
    'Bonus': ['N64 Rainbow Road'] + list(TRACKS[-16:]),
    '32': pad + list(TRACKS[:32]),
    '48': pad + list(TRACKS)
}




import MK8D.constants as cnst
from datetime import datetime
from datetime import timedelta


###############################################################################
# Timing
###############################################################################
REFT = datetime(1900, 1, 1, 0, 0, 0, 0)


def tdeltaToSec(tDelta, microSec=1000000):
    """Converts a timedelta object to a seconds equivalent

    Args:
        tDelta (timedelta): Timedelta object to convert to seconds
        microSec (int, optional): Microseconds fractional. Defaults to 1000000.

    Returns:
        float: Equivalent time in seconds.
    """
    tm = (tDelta.seconds) + (tDelta.microseconds / microSec)
    return tm


def tStrToSecs(tStr, refTime=REFT):
    """Converts a time string into a seconds float

    Args:
        tStr (string): Time string in the form H:M:S.ms
        refTime ([datetime], optional): Internal constant to convert times. Defaults to REFT.

    Returns:
        float: Equivalent time in seconds.
    """
    trackTiming = datetime.strptime(tStr[:-1], '%H:%M:%S.%f')
    tDiff = tdeltaToSec(trackTiming - REFT)
    return tDiff


def secToMin(sec, prec=cnst.TPREC):
    return timedelta(seconds=sec)


def minsToHr(mins, prec=cnst.TPREC):
    return timedelta(minutes=mins)


def scaleDevs(x, tDevs):
    return (x - min(tDevs)) / (max(tDevs) - min(tDevs))




import pandas as pd
from xmltodict import parse
from datetime import datetime
from collections import OrderedDict
import MK8D.auxiliary as aux


###############################################################################
# XML Dictionary 
###############################################################################
def getDictFromXMLFile(filePath):
    with open(filePath) as fd:
        doc = parse(fd.read())
    return doc


def getSegments(doc):
    return doc['Run']['Segments']['Segment']


def getSpeed(doc):
    sp = doc['Run']['Metadata']['SpeedrunComVariables']['Variable'][0]['#text']
    return sp


def getVersion(doc):
    vs = doc['Run']['Metadata']['SpeedrunComVariables']['Variable'][1]['#text']
    return vs


def getItems(doc):
    it = doc['Run']['Metadata']['SpeedrunComVariables']['Variable'][2]['#text']
    return it


def getCategory(doc):
    ct = doc['Run']['CategoryName']
    return ct


###############################################################################
# Runs dictionary
###############################################################################
def parseRunsFromFile(filePath, metadata=True):
    # Load file into dictionary and get segments
    doc = getDictFromXMLFile(filePath)
    segments = getSegments(doc)
    # Get metadata
    (speed, version, category, items) = [None] * 4
    if metadata:
        (speed, version, items, category) = (
            getSpeed(doc), getVersion(doc),
            getItems(doc), getCategory(doc)
        )
    # Assemble output dictionary
    runs = {
        'category': category,
        'version': version,
        'speed': speed,
        'items': items,
        'segments': segments
    }
    return runs


def getSegmentsNames(segments):
    return [i['Name'] for i in segments]


def getRunsDict(runs):
    (nms, tracks) = (getSegmentsNames(runs['segments']), OrderedDict())
    for (ix, track) in enumerate(runs['segments']):
        tHist = track['SegmentHistory']['Time']
        tEntry = {int(i['@id']): aux.tStrToSecs(i['RealTime']) for i in tHist}
        tracks.update({nms[ix]: tEntry})
    return tracks


def getRIDFromFile(file, prependID='', zfill=5):
    doc = getDictFromXMLFile(file)
    history = doc['Run']['AttemptHistory']['Attempt']
    getAttemptsDates = [
        (i['@id'], datetime.strptime(i['@started'][:-3], '%m/%d/%Y %H:%M')) 
        for i in history
    ]
    strFmt = '%-y/%m/%d'
    ids = {
        i[0]: '{} ({}:{})'.format(
            i[1].strftime(strFmt),
            prependID, str(i[0]).zfill(zfill)
        ) 
        for i in getAttemptsDates
    }
    return ids


###############################################################################
# Dataframe
###############################################################################
def getTrackList(runs, tracks, rid, name, prependID='', zfill=5):
    # Constant data and track info
    (spd, itm, cat, ver) = (
        runs['speed'], runs['items'], runs['category'], runs['version']
    )
    track = tracks[name]
    # Run ID with timings
    (ids, times) = (list(track.keys()), list(track.values()))
    # If key, add date from dictionary, else just add the normal
    trackList = []
    for (key, time) in zip(ids, times):
        if str(key) in rid:
            entry = (rid[str(key)], name, time, ver, itm, spd, cat)
        else:
            entry = (
                '({}:{})'.format(prependID, str(key).zfill(zfill)),
                name, time, ver, itm, spd, cat
            )
        trackList.append(entry)
    return trackList


def getRunsDataframe(runs, tracks, rid, prependID='', zfill=5):
    tracksNames = list(tracks.keys())
    tracksList = []
    for i in tracksNames:
        tracksList.extend(
            getTrackList(runs, tracks, rid, i, prependID=prependID, zfill=zfill)
        )
    columns = ['ID', 'Track', 'Time', 'Version', 'Items', 'Speed', 'Category']
    runsDataframe = pd.DataFrame(tracksList, columns=columns)
    return runsDataframe


def getRunsDataframeFromFile(file, metadata=True, prependID='', zfill=5):
    runs = parseRunsFromFile(file, metadata=metadata)
    rid = getRIDFromFile(file, prependID=prependID, zfill=zfill)
    trks = getRunsDict(runs)
    data = getRunsDataframe(runs, trks, rid, prependID=prependID, zfill=zfill)
    return data


def compileRunsDataframeFromFiles(
    filesList, metadata=True, prependID=True, zfill=3
):
    if prependID:
        dfs = [
            getRunsDataframeFromFile(
                file, 
                metadata=metadata, 
                prependID=str(i+1).zfill(2),
                zfill=3
            ) 
            for (i, file) in enumerate(filesList)
        ]
    else:
        dfs = [
            getRunsDataframeFromFile(
                file, metadata=metadata, prependID='', zfill=3
            ) 
            for (i, file) in enumerate(filesList)
        ]
    df = pd.concat(dfs)
    return df



import numpy as np
import pandas as pd
import MK8D.constants as cnst
pd.options.mode.chained_assignment = None


def getFinishedRunsID(data, tracks=cnst.TRACKS):
    runsIds = []
    for i in tracks:
        filtr = (data['Track']== i)
        runsIds.append(set(data['ID'][filtr]))
    finishedRuns = set.intersection(*runsIds)
    return finishedRuns


def getFinishedRuns(data, tracks=cnst.TRACKS):
    fshdRuns = getFinishedRunsID(data, tracks)
    fltr = (data['ID'].isin(fshdRuns), data['Track'].isin(set(tracks)))
    fltrBool = [all(i) for i in zip(*fltr)]
    return data[fltrBool]


def getTrackTime(data, rid, track):
    row = data[data['ID'] == rid]
    return float(row[row['Track'] == track]['Time'])


def getRunByID(data, rid):
    return data[(data['ID'] == rid)]


def convertRunCTime(runData, tracks=cnst.TRACKS):
    rid = list(runData['ID'])[0]
    runData['Track'] = pd.Categorical(
        runData.Track, categories=tracks, ordered=True
    )
    runData = runData.sort_values(by='Track')
    cTime = np.cumsum([getTrackTime(runData, rid, track) for track in tracks])
    runData['Time'] = cTime
    return runData


def convertFinishedRunsToCTimes(data, fshdRIds, tracks=cnst.TRACKS):
    fshdRunsIDList = list(fshdRIds)
    runsCTimesDataList = []
    for rid in fshdRunsIDList:
        runData = getRunByID(data, rid)
        runDataCTime = convertRunCTime(runData, tracks=tracks)
        runsCTimesDataList.append(runDataCTime)
    runsCTimes = pd.concat(runsCTimesDataList)
    return runsCTimes


def centerTrackTimes(times, centerFunction=np.mean):
    offset = centerFunction(times)
    offsetTimes = [(i - offset) for i in times]
    return (offsetTimes, offset)


def centerRunsCTimes(runsCTimes, centerFunction=np.mean):
    cName = centerFunction.__name__
    # Get unique run ID and tracks names 
    (ids, tracks) = (
        list(runsCTimes['ID'].unique()), 
        list(runsCTimes['Track'].unique())
    )
    # Create copy to store modified dataframe
    runsCTimesC = runsCTimes.copy()
    offsets = []
    # Iterate through tracks to get the "center" and shift times around it
    for track in tracks:
        trackTimes = [getTrackTime(runsCTimes, i, track) for i in ids]
        (centered, center) = centerTrackTimes(
            trackTimes, centerFunction=centerFunction
        )
        offsets.append(center)
        # Replace centered time in the copied dataframe
        for (j, time) in enumerate(centered):
            fltr = (runsCTimesC['Track'] == track, runsCTimes['ID'] == ids[j])
            selector = [all(i) for i in zip(*fltr)]
            runsCTimesC[selector]['Time'] = ids[j]
            runsCTimesC.loc[selector, 'Center ' + cName] = center
            runsCTimesC.loc[selector, 'Center offset'] = time
    return runsCTimesC


def convertTimeFromSec(data, timeTarget='Hours'):
    dataTemp = data.copy()
    if timeTarget == 'Hours':
        timeFun = lambda x: x / (60 * 60)
    elif timeTarget == 'Minutes':
        timeFun = lambda x: x / 60
    else:
        return 'Available options are: Hours and Minutes'
    dataTemp['Time'] = dataTemp['Time'].apply(lambda x: timeFun(x))
    return dataTemp


def timeBetweenTracks(data, rid, start, end):
    runp = data[data['ID']==rid]
    (stpt, strt) = [
        float(runp[runp['Track'] == i]['Time']) for i in (end, start)
    ]
    interval = stpt - strt
    return interval


def runsCategoryTimes(data, start, end):
    ids = list(data['ID'].unique())
    # Populate times list ---------------------------------------------------------
    times = []
    for rid in ids:
        interval = timeBetweenTracks(data, rid, start, end)
        times.append(interval)
    return times



import os
import setuptools
from version import version as this_version

this_directory =  os.path.abspath(os.path.dirname(__file__))
version_path = os.path.join(this_directory, 'MK8D', '_version.py')
with open(version_path, 'wt') as fversion:
    fversion.write('__version__ = "'+this_version+'"')


REQUIRED_PACKAGES=[
    'matplotlib>=3.3.2', 'xmltodict>=0.12.0',
    'pandas>=1.1.4', 'numpy>=1.19.4', 'plotly>=4.13.0',
    'plotly-express>=0.4.1', 'colour>=0.1.5'
]


with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="MK8D",                                                # name of project
    install_requires=REQUIRED_PACKAGES,                         # all requirements used by this package
    version=this_version,                                       # project version, read from version.py
    author="chipdelmal",                                        # Author, shown on PyPI
    scripts=[                                                   # command line scripts installed
        './bin/MK8D_trk2csv',
        './bin/MK8D_run2csv'
    ],                           
    author_email="chipdelmal@gmail.com",                        # Author email
    description="Mario Kart 8 Deluxe livesplit analyzer",       # Short description of project
    long_description=long_description,                          # Long description, shown on PyPI
    long_description_content_type="text/markdown",              # Content type. Here, we used a markdown file.
    url="https://github.com/Chipdelmal/MK8D",                   # github path
    packages=setuptools.find_packages(),                        # automatically finds packages in the current directory. You can also explictly list them.
    classifiers=[                                               # Classifiers give pip metadata about your project. See https://pypi.org/classifiers/ for a list of available classifiers.
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',                                    # python version requirement
)



from colour import Color

def generateColorSwatch(
    baseColor, levels,
    alphaOffset=(.1, .9), lumaOffset=(.1, 1),
):
    colorsList = [None] * levels
    for i in range(levels):
        c = Color(baseColor)
        baseLum = lumaOffset[1] - c.get_luminance()
        lumLevel = (baseLum - ((lumaOffset[1]-baseLum)/levels * i)) + (lumaOffset[0])
        c.set_luminance(lumLevel)
        rgb = c.get_rgb()
        alphaLevel = alphaOffset[0] + ((alphaOffset[1]-alphaOffset[0]) * i/levels)
        colorsList[i] = (rgb[0], rgb[1], rgb[2], alphaLevel)
    return colorsList